﻿using System;
using System.IO;
using System.Threading.Tasks;
using Android.Bluetooth;
using Android.Content;
using Android.Util;
using TFM.Bluetooth;
using TFM.Droid;
using Xamarin.Forms;

[assembly: Xamarin.Forms.Dependency(typeof(TFM.Droid.BluetoothManager))]
namespace TFM.Droid
{
	public class BluetoothManager: IBluetoothManager
	{
        object locker = new object();
		public BluetoothManager()
		{
		}

        /// <summary>
        /// Connect this instance.
        /// </summary>
        /// <returns>The connect.</returns>
        public Task<bool> Connect()
        {
			// Get the MainActivity instance
			MainActivity activity = Forms.Context as MainActivity;

            activity.ConnectTaskCompletionSource = new TaskCompletionSource<bool>();
            activity.Connect();

			// Return Task object
            return activity.ConnectTaskCompletionSource.Task;
        }

        /// <summary>
        /// Gets the file.
        /// </summary>
        /// <returns>The file.</returns>
        public string GetFile()
        {
			var documentsPath = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
			var filePath = Path.Combine(documentsPath, "InverterCopy.xml");
			return System.IO.File.ReadAllText(filePath);
        }

        /// <summary>
        /// Saves the file.
        /// </summary>
        /// <returns><c>true</c>, if file was saved, <c>false</c> otherwise.</returns>
        /// <param name="xDoc">X document.</param>
        public bool SaveFile(string xDoc)
        {
			var documentsPath = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
			var filePath = Path.Combine(documentsPath, "InverterCopy.xml");
            try
            {
                System.IO.File.WriteAllText(filePath, xDoc);
                return true;
            }
            catch (Exception)
            {
                return false;
            }

        }

        /// <summary>
        /// Transfers the data.
        /// </summary>
        /// <returns>The data.</returns>
        /// <param name="dataToSend">Data to send.</param>
        public Task<byte[]> TransferData(byte[] dataToSend)
		{
            lock(locker)
            {
				// Get the MainActivity instance
				MainActivity activity = Forms.Context as MainActivity;

				// Save the TaskCompletionSource object as a MainActivity property
				activity.TransferDataTaskCompletionSource = new TaskCompletionSource<byte[]>();

				activity.Transfer(2, dataToSend);

				// Return Task object
				return activity.TransferDataTaskCompletionSource.Task;
            }
			
		}

		
	}
}
