﻿using System;

using Android.App;
using Android.Content;
using Android.Content.PM;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;
using Android.Bluetooth;
using System.Linq;
using Android.Util;
using Java.Util;
using Java.Lang;
using System.Threading.Tasks;

namespace TFM.Droid
{
	[Activity(Label = "TFM.Droid", Icon = "@drawable/icon", Theme = "@style/MyTheme", MainLauncher = true, ConfigurationChanges = ConfigChanges.ScreenSize | ConfigChanges.Orientation)]
	public class MainActivity : global::Xamarin.Forms.Platform.Android.FormsAppCompatActivity
	{
		// Debugging
		private const string TAG = "BluetoothService";
		private const bool Debug = true;

		// Message types sent from the BluetoothChatService Handler
		// TODO: Make into Enums
		public const int MESSAGE_STATE_CHANGE = 1;
		public const int MESSAGE_READ = 2;
		public const int MESSAGE_WRITE = 3;
		public const int MESSAGE_DEVICE_NAME = 4;
		public const int MESSAGE_TOAST = 5;

		// Key names received from the BluetoothChatService Handler
		public const string DEVICE_NAME = "device_name";
		public const string TOAST = "toast";

		// Local Bluetooth adapter
		private BluetoothAdapter bluetoothAdapter = null;
		// Member object for the chat services
        private BluetoothTransferService transferService = null;

        private static UUID MY_UUID = UUID.FromString("3b29c7a5-13b5-42f3-8fb7-8cb80f35aa68");
        private static UUID PUBLIC_UUID = UUID.FromString("6332a546-d1b2-bfb8-594f-bf1fa614bc8f");

		private BluetoothServerSocket mmServerSocket;

		public TaskCompletionSource<bool> PickImageTaskCompletionSource { set; get; }

        public TaskCompletionSource<bool> ConnectTaskCompletionSource { set; get; }

        public TaskCompletionSource<byte[]> TransferDataTaskCompletionSource { set; get; }

        private object locker = new object();

        private MyHandler myHandler;


		protected override void OnCreate(Bundle bundle)
		{
        
            base.OnCreate(bundle);

			global::Xamarin.Forms.Forms.Init(this, bundle);

			// Get local Bluetooth adapter
			bluetoothAdapter = BluetoothAdapter.DefaultAdapter;
	
			// If the adapter is null, then Bluetooth is not supported
			if (bluetoothAdapter == null) {
				Toast.MakeText (this, "Bluetooth is not available", ToastLength.Long).Show ();
				Finish ();
				return;
			}

			LoadApplication(new App());
            this.myHandler = new MyHandler(this);
		}

		protected override void OnStart()
		{
			base.OnStart();
            SetupService();
        }

        private void SetupService()
		{
			Log.Debug(TAG, "SetupService()");

			// Initialize the BluetoothTransferService to perform bluetooth connections
			transferService = new BluetoothTransferService();
		}




        public void Connect()
        {
            BluetoothDevice device = null;

            foreach (var item in bluetoothAdapter.BondedDevices)
            {
                foreach (var UUid in item.GetUuids())
                {
                    
                    if (UUid.ToString() == PUBLIC_UUID.ToString())
                    {
                        device = item;
                        break;
                    }
                }

                if (device != null)
                {
                    break;
                }
            }

            if (device == null)
            {
                device = bluetoothAdapter.BondedDevices.FirstOrDefault(x => x.Name.Contains("INVERTER"));
            }

            if(device != null)
            {
				// Attempt to connect to the device
				transferService.Connect(device);

                // Check if the connection was ok
                if(transferService.GetState() == BluetoothTransferService.STATE_CONNECTED)
                {
                    this.SendMessage("Connected to the inverter");
                    ConnectTaskCompletionSource.SetResult(true);
                }
                else
                {
					this.SendMessage("Can not connect to the inverter");
					ConnectTaskCompletionSource.SetResult(false);
                }

				
            }
            else
            {
                this.SendMessage("Can not connect to the inverter");
                //Toast.MakeText(this, "Can not connect to the inverter", ToastLength.Short);
                ConnectTaskCompletionSource.SetResult(false);
            }
			
        }

        private void SendMessage(string text)
        {
			Message msg = myHandler.ObtainMessage();
			Bundle bundle = new Bundle();
			bundle.PutString(TOAST, text);
			msg.Data = bundle;
			myHandler.SendMessage(msg);
        }

        public void Transfer(int requestCode, byte[] dataToSend)
        {
            lock (locker)
            {
                switch (requestCode)
                {
                    case 2:
                        // Attempt to connect to the device
                        byte[] response = transferService.Write(dataToSend);
                        TransferDataTaskCompletionSource.SetResult(response);
                        break;
                }
            }
			
        }

		public override bool OnCreateOptionsMenu(IMenu menu)
		{
			//var inflater = MenuInflater;
			//inflater.Inflate(Resource.Menu.option_menu, menu);
			return true;
		}

		public override bool OnOptionsItemSelected(IMenuItem item)
		{
			
			return false;
		}

		// The Handler that gets information back from the BluetoothChatService
		private class MyHandler : Handler
		{
            MainActivity bluetoothActivity;

			public MyHandler(MainActivity activity)
			{
				bluetoothActivity = activity;
			}

            public override void HandleMessage(Message msg)
            {
                Toast.MakeText(Application.Context, msg.Data.GetString(TOAST), ToastLength.Short).Show();

            }
		}


	}


}


