﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Linq;
using TFM.Model;
using Xamarin.Forms;

namespace TFM.Bluetooth
{
    public class InverterConnectionManager: IInverterConnectionManager
    {
        /// <summary>
        /// Gets the model.
        /// </summary>
        /// <returns>The model.</returns>
		public async Task<string> GetModel()
		{

			byte[] model = null;
			await Task.Run(async () =>
		 {


			 List<byte> request = new List<byte>();
			 request.Add(2);
			 byte[] indexArray = BitConverter.GetBytes((ushort)1);
			 request.AddRange(indexArray);
			 byte[] subindexArray = BitConverter.GetBytes((ushort)1);
			 request.AddRange(subindexArray);
			 byte[] dataArray = BitConverter.GetBytes((int)0);
			 request.AddRange(dataArray);

			 model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
		 });

			return Encoding.UTF8.GetString(model, 1, model.Length - 1);

		}

        /// <summary>
        /// Reads the monitor parameter value.
        /// </summary>
        /// <param name="monitorParameter">Monitor parameter.</param>
        public async void ReadMonitorParameterValue(MonitorParameter monitorParameter)
        {
            
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(2);
                 ushort index = Convert.ToUInt16(monitorParameter.Index);
                 ushort subindex = Convert.ToUInt16(monitorParameter.Subindex);
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                 byte[] dataArray = BitConverter.GetBytes((int)0);
                 request.AddRange(dataArray);

                 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
                 if (model[0] == 0)
                 {
                     int parameterValue = BitConverter.ToInt32(model, 1);
                     monitorParameter.Value = parameterValue;
                 }
                 else
                 {
                     // Error transfering
                 }
             });
        }

        /// <summary>
        /// Reads the alarm.
        /// </summary>
        /// <returns>The alarm.</returns>
        public async Task<int> ReadAlarm()
        {
            
            byte[] model = null;
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(2);
                 ushort index = 1000;
                 ushort subindex = 1;
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                 byte[] dataArray = BitConverter.GetBytes((int)0);
                 request.AddRange(dataArray);

                 model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
             });

            return BitConverter.ToInt32(model, 1);
        }

        /// <summary>
        /// Reads the parameter value.
        /// </summary>
        /// <returns>The parameter value.</returns>
        /// <param name="parameter">Parameter.</param>
        public async Task<bool> ReadParameterValue(Parameter parameter)
        {
            bool error = false;
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(2);
                 ushort index = Convert.ToUInt16(parameter.Index);
                 ushort subindex = Convert.ToUInt16(parameter.Subindex);
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                 byte[] dataArray = BitConverter.GetBytes((int)0);
                 request.AddRange(dataArray);

                 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
                 if (model[0] == 0)
                 {
                     int parameterValue = BitConverter.ToInt32(model, 1);
                     parameter.Value = parameterValue;
                 }
                 else
                 {
                     error = true;
                     // Error transfering
                 }
             });

            return error;
        }

        /// <summary>
        /// Clears the alarm.
        /// </summary>
        public async void ClearAlarm()
        {
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(3);
                 ushort index = 1000;
                 ushort subindex = 1;
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                 byte[] dataArray = BitConverter.GetBytes((int)0);
                 request.AddRange(dataArray);

                 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
             });
			}
 
        /// <summary>
        /// Writes the parameter value.
        /// </summary>
        /// <returns>The parameter value.</returns>
        /// <param name="parameter">Parameter.</param>
        public async Task<bool> WriteParameterValue(Parameter parameter)
        {
            bool error = false;
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(3);
                 ushort index = Convert.ToUInt16(parameter.Index);
                 ushort subindex = Convert.ToUInt16(parameter.Subindex);
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                byte[] dataArray = BitConverter.GetBytes((int)parameter.Value);
                 request.AddRange(dataArray);

                 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
             });
			
            return error;
        }

        /// <summary>
        /// Executes the run operation.
        /// </summary>
        public async void Run()
        {
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(3);
                 ushort index = 10;
                 ushort subindex = 1;
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                 byte[] dataArray = BitConverter.GetBytes((int)1);
                 request.AddRange(dataArray);

                 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
             });
        }

        /// <summary>
        /// Executes the stop function
        /// </summary>
        public async void Stop()
        {
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(3);
                 ushort index = 2000;
                 ushort subindex = 1;
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                 byte[] dataArray = BitConverter.GetBytes((int)0);
                 request.AddRange(dataArray);

                 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
             });
        }

        /// <summary>
        /// Executes the initialize function.
        /// </summary>
        public async void Initialize()
        {
            await Task.Run(async () =>
             {
                 List<byte> request = new List<byte>();
                 request.Add(5);
                 ushort index = 1;
                 ushort subindex = 1;
                 byte[] indexArray = BitConverter.GetBytes(index);
                 request.AddRange(indexArray);
                 byte[] subindexArray = BitConverter.GetBytes(subindex);
                 request.AddRange(subindexArray);
                 byte[] dataArray = BitConverter.GetBytes((int)0);
                 request.AddRange(dataArray);

                 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
             });
        }

        /// <summary>
        /// Saves an internal copy of the parameters
        /// </summary>
        public async void Copy()
        {
            IEnumerable<Parameter> parameters = await InverterModelFactory.Instance.GetParameters();
            XDocument doc = new XDocument();
            XElement xParameters = new XElement("Parameters");

            foreach (var parameter in parameters)
            {
                bool error = await this.ReadParameterValue(parameter);
                xParameters.Add(new XElement("Parameter", new XAttribute("Index", parameter.Index), new XAttribute("Subindex", parameter.Subindex), new XAttribute("Value", parameter.Value)));
            }

            doc.Add(xParameters);

            bool result = DependencyService.Get<IBluetoothManager>().SaveFile(doc.ToString());
			}

        /// <summary>
        /// Apply the remembered parameters.
        /// </summary>
        public async void Apply()
        {
            IEnumerable<Parameter> parameters = await InverterModelFactory.Instance.GetParameters();
            string fileText = DependencyService.Get<IBluetoothManager>().GetFile();
            XDocument doc = XDocument.Parse(fileText);
            foreach(var xParameter in doc.Root.Descendants("Parameter"))
            {
                string index = xParameter.Attribute("Index").Value;
                string subindex = xParameter.Attribute("Subindex").Value;
                int parameterValue = Convert.ToInt32(xParameter.Attribute("Value").Value);

                Parameter parameter = parameters.FirstOrDefault( p => p.Index == index && p.Subindex == subindex);
                if(parameter != null)
                {
                    parameter.Value = parameterValue;
                    bool error = await this.WriteParameterValue(parameter);
                }
            }

        }

        /// <summary>
        /// Reads the information of the inverter.
        /// </summary>
        /// <returns>The information.</returns>
        /// <param name="information">Information.</param>
        public async Task<bool> ReadInformation(Information information)
        {
			bool error = false;
            await Task.Run(async () =>
			 {
				 List<byte> request = new List<byte>();
				 request.Add(2);
                ushort index = Convert.ToUInt16(information.Index);
                ushort subindex = Convert.ToUInt16(information.Subindex);
				 byte[] indexArray = BitConverter.GetBytes(index);
				 request.AddRange(indexArray);
				 byte[] subindexArray = BitConverter.GetBytes(subindex);
				 request.AddRange(subindexArray);
				 byte[] dataArray = BitConverter.GetBytes((int)0);
				 request.AddRange(dataArray);

				 byte[] model = await DependencyService.Get<IBluetoothManager>().TransferData(request.ToArray());
				 if (model[0] == 0)
				 {
                    information.Value = Encoding.UTF8.GetString(model, 1, model.Length - 1);
				 }
				 else
				 {
					 error = true;
					 // Error transfering
				 }
			 });

			return error;
        }
    }
}
