﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Threading.Tasks;
using TFM.Bluetooth;
using TFM.Model;
using Xamarin.Forms;

namespace TFM
{
	public partial class ConnectPage : ContentPage
	{
		private readonly IConnectManager connectManager;
		private readonly ConnectViewModel connectViewModel;

		public ConnectPage(IConnectManager connectManager)
		{
			this.connectManager = connectManager;
			this.connectViewModel = new ConnectViewModel();
			this.BindingContext = this.connectViewModel;
            //Image icon = new Image()
            //{
              //  Source = ImageSource.FromFile("icon.png"),
               // Aspect = Aspect.AspectFit
            //};

            //this.IconImage = icon;
			InitializeComponent();
            this.IconImage.Source = ImageSource.FromFile("invertericon.png");
		}

		public ConnectPage(): this(null)
		{
		}

		private async void OnConnect(object sender, EventArgs e)
		{
			this.connectViewModel.IsVisible = true;
            IInverterConnectionManager manager = new InverterConnectionManager();

            bool connected = false;

            await Task.Run(async () =>
	           {
                   connected = await DependencyService.Get<IBluetoothManager>().Connect();
	           });  

            if(connected)
            {string model = await manager.GetModel();

				// Use model to select the inverter
				InverterModelFactory.Instance.Initialize(model);
				Task<IEnumerable<Parameter>> parametersTask = InverterModelFactory.Instance.GetParameters();
				IEnumerable<Parameter> parameterList = await parametersTask;
				Task<IEnumerable<MonitorParameter>> monitorParametersTask = InverterModelFactory.Instance.GetMonitorParameters();
				IEnumerable<MonitorParameter> monitorParameterList = await monitorParametersTask;
				Task<IEnumerable<Alarm>> alarmsTask = InverterModelFactory.Instance.GetAlarms();
				IEnumerable<Alarm> alarms = await alarmsTask;
				this.connectManager.ShowMainPage();
            }
            else
            {
                this.connectViewModel.IsVisible = false;
            }

			
		}
	}
}
