﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using TFM.XML;

namespace TFM.Model
{
	public class InverterModelFactory
	{
		private static InverterModelFactory instance;
		private static readonly object padlock = new object();
		private readonly XMLReader xmlReader;

		private IEnumerable<Parameter> parameters;
		private IEnumerable<MonitorParameter> monitorParameters;
		private IEnumerable<Alarm> alarms;
		private IList<Information> informations;

		private InverterModelFactory()
		{
			xmlReader = new XMLReader();
		}

		public static InverterModelFactory Instance
		{
			get
			{
				lock (padlock)
				{
					if (instance == null)
					{
						instance = new InverterModelFactory();
					}
					return instance;
				}
			}
		}

		public void Initialize(string model)
		{
			this.xmlReader.Initialize(model);
		}

		public async Task<IEnumerable<Parameter>> GetParameters()
		{
			if (parameters == null)
			{
				Task<IEnumerable<Parameter>> parametersTask = xmlReader.ReadParameters();
				parameters = await parametersTask;
			}

			return parameters;
		}

		public async Task<IEnumerable<MonitorParameter>> GetMonitorParameters()
		{
			if (monitorParameters == null)
			{
				Task<IEnumerable<MonitorParameter>> parametersTask = xmlReader.ReadMonitorParameters();
				monitorParameters = await parametersTask;
			}

			return monitorParameters;
		}

		public async Task<IEnumerable<Alarm>> GetAlarms()
		{
			if (alarms == null)
			{
				Task<IEnumerable<Alarm>> alarmTask = xmlReader.ReadAlarms();
				alarms = await alarmTask;
			}

			return alarms;
		}

		public IEnumerable<Information> GetInformations()
		{
			if (informations == null)
			{
				informations = new List<Information>();
				Information modelNameInformation = new Information() { Index = "9000", Subindex = "01", Name = "Model Name", Value = "Dummy Inverter" };
				Information firmwareInformation = new Information() { Index = "9000", Subindex = "02", Name = "Firmware Number", Value = "1.1x" };
				Information serialNumberInformation = new Information() { Index = "9000", Subindex = "03", Name = "Serial Number", Value = "00000000" };
				Information motorInformation = new Information() { Index = "9000", Subindex = "04", Name = "Motor Information", Value = "Motor 1" };
				informations.Add(modelNameInformation);
				informations.Add(firmwareInformation);
				informations.Add(serialNumberInformation);
				informations.Add(motorInformation);
			}

			return informations;
		}
	}
}
