﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using TFM.Bluetooth;
using TFM.Model;
using Xamarin.Forms;

namespace TFM
{
	public partial class MonitorList : ContentPage
	{
		private IList<MonitorParameter> parameterList;
		TimeSpan seconds = TimeSpan.FromSeconds(3);
		bool visible = false;
		bool monitoring = false;
        IInverterConnectionManager manager = new InverterConnectionManager();

		public MonitorList()
		{
			this.InitializeView();
			InitializeComponent();
		}

		private async void InitializeView()
		{
			Task<IEnumerable<MonitorParameter>> parametersTask = InverterModelFactory.Instance.GetMonitorParameters();
			IEnumerable<MonitorParameter> monitorParameterList = await parametersTask;
			parameterList = monitorParameterList.ToList();

			ObservableCollection<MonitorParameterViewModel> parameterViewModels = new ObservableCollection<MonitorParameterViewModel>();

			foreach (MonitorParameter parameter in parameterList)
			{
				MonitorParameterViewModel parameterViewModel = new MonitorParameterViewModel(parameter);
				parameterViewModels.Add(parameterViewModel);
			}

			MonitorParameterListViewModel monitorParameterListViewModel = new MonitorParameterListViewModel();
			monitorParameterListViewModel.ParameterViewModels = parameterViewModels;

			this.BindingContext = monitorParameterListViewModel;

			this.StartMonitor();

			monitoring = true;
		}

		protected override void OnAppearing()
		{
			base.OnAppearing();
			this.visible = true;
			if (monitoring == false)
			{
				this.StartMonitor();
				monitoring = true;
			}

		}

		protected override void OnDisappearing()
		{
			base.OnDisappearing();
			this.visible = false;
			this.monitoring = false;
		}

		private void StartMonitor()
		{
			Device.StartTimer(seconds, () =>
			{
				foreach (MonitorParameter monitorParameter in parameterList)
				{
                    this.manager.ReadMonitorParameterValue(monitorParameter);

				}

				return this.visible;
			});
		}
	}
}
