﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading.Tasks;
using TFM.Model;
using Xamarin.Forms;

namespace TFM
{
	public class MonitorParameterViewModel : INotifyPropertyChanged
	{
		private readonly MonitorParameter model;

		public MonitorParameterViewModel(MonitorParameter parameter)
		{
			this.model = parameter;
			this.model.PropertyChanged += this.OnModelPropertyChanged;
		}
		public event PropertyChangedEventHandler PropertyChanged;

		public string ParameterName
		{
			get
			{
				return model.Name;
			}
		}

		public int? ParameterValue
		{
			set
			{
				if (this.model.Value != value)
				{
					this.model.Value = value;
					OnPropertyChanged("ParameterValue");
				}
			}
			get
			{
				return this.model.Value;
			}
		}

		public string ParameterDescription
		{
			get
			{
				return model.Description;
			}
		}

		public string ParameterUnit
		{
			get
			{
				return model.Unit;
			}
		}

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this,
					new PropertyChangedEventArgs(propertyName));
			}
		}

		// When property gets changed in the Model, raise the PropertyChanged 
		// event of the ViewModel copy of the property
		private void OnModelPropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			if (e.PropertyName == "Value")
			{
                OnPropertyChanged("ParameterValue");
			}
		}
	}
}