﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using TFM.Bluetooth;
using TFM.Model;
using Xamarin.Forms;

namespace TFM
{
	public partial class StatusPage : ContentPage
	{
        IEnumerable<Alarm> alarmList;
		AlarmViewModel viewModel;
		TimeSpan seconds = TimeSpan.FromSeconds(3);
		bool visible = false;
		bool monitoring = false;
        IInverterConnectionManager manager = new InverterConnectionManager();

		public StatusPage()
		{
            InitializeView();
			InitializeComponent();
		}

		private async void InitializeView()
		{
			Task<IEnumerable<Alarm>> alarmTask = InverterModelFactory.Instance.GetAlarms();
			this.alarmList = await alarmTask;

			AlarmViewModel alarmViewModel = new AlarmViewModel();
            alarmViewModel.Name = string.Empty;
            alarmViewModel.Details = string.Empty;

			this.viewModel = alarmViewModel;

			this.BindingContext = alarmViewModel;
		}

		private void OnClear(object sender, EventArgs e)
		{
            this.manager.ClearAlarm();
		}

		private async void OnClick(object sender, EventArgs e)
		{
			await Navigation.PushAsync(new MonitorList());
		}

		protected override void OnAppearing()
		{
			base.OnAppearing();
			this.visible = true;
			if (monitoring == false)
			{
			    this.StartMonitor();
				monitoring = true;
			}

		}

		protected override void OnDisappearing()
		{
			base.OnDisappearing();
			this.visible = false;
			this.monitoring = false;
		}

		private void StartMonitor()
		{
			Device.StartTimer(seconds, () =>
			{
                Task.Factory.StartNew(async () =>
				{
					// Do the actual request and wait for it to finish.
					int eventCode = await this.manager.ReadAlarm();
					// Switch back to the UI thread to update the UI
					Device.BeginInvokeOnMainThread(() =>
						{
	                        if(eventCode == 0)
	                        {
	                            this.viewModel.Name = string.Empty;
	                            this.viewModel.Details = string.Empty;
	                        }
                            else
                            {
								Alarm alarm = this.alarmList.FirstOrDefault(x => x.EventCode == eventCode);
								this.viewModel.Name = alarm.Name;
								this.viewModel.Details = alarm.Details;
                            }
                            
	                        if(this.visible)
	                        {
                                StartMonitor();
                            }
                        });
				});

				// Don't repeat the timer (we will start a new timer when the request is finished)
				return false;
			});
		}

	}
}
