﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Linq;
using TFM.Model;
using System.Reflection;
using System.IO;

namespace TFM.XML
{
	public class XMLReader
	{
		private string path;
        Random rnd = new Random();

		public void Initialize(string model)
		{
			this.path = "TFM.XML." + model + ".xml";
		}

		public async Task<IEnumerable<Parameter>> ReadParameters()
		{
			List<Parameter> rawData = null;
			var assembly = typeof(XMLReader).GetTypeInfo().Assembly;
			Stream stream = assembly.GetManifestResourceStream(this.path);

			await Task.Factory.StartNew(delegate
			{
				XDocument doc = XDocument.Load(stream);
                IEnumerable<Parameter> parameters = from s in doc.Root.Elements("Parameters").Elements("Configuration").Descendants("Parameter")
                                                    select new Parameter
                                                    {
                                                        Index = s.Attribute("Index").Value,
                                                        Subindex = s.Attribute("Subindex").Value,
                                                        Min = Convert.ToInt32(s.Attribute("Min").Value),
                                                        Max = Convert.ToInt32(s.Attribute("Max").Value),
                                                        Default = Convert.ToInt32(s.Attribute("Default").Value),
                                                        Unit = s.Attribute("Unit").Value,
                                                        Name = s.Element("Name").Value,
                                                        Help = s.Element("Help").Value,
                                                        Description = s.Element("Description").Value
													};

				rawData = parameters.ToList();
			});

			return rawData;
		}

		public async Task<IEnumerable<MonitorParameter>> ReadMonitorParameters()
		{
			List<MonitorParameter> rawData = null;
			var assembly = typeof(XMLReader).GetTypeInfo().Assembly;
			Stream stream = assembly.GetManifestResourceStream(this.path);

			await Task.Factory.StartNew(delegate
			{
				XDocument doc = XDocument.Load(stream);
                IEnumerable<MonitorParameter> parameters = from s in doc.Root.Elements("Parameters").Elements("Monitor").Descendants("Parameter")
                                                           select new MonitorParameter
                                                           {
                                                               Index = s.Attribute("Index").Value,
                                                               Subindex = s.Attribute("Subindex").Value,
                                                               Unit = s.Attribute("Unit").Value,
                                                               Name = s.Element("Name").Value,
                                                               Description = s.Element("Description").Value
                                                                              
														   };

				rawData = parameters.ToList();
			});

			return rawData;
		}

		public async Task<IEnumerable<Alarm>> ReadAlarms()
		{
			List<Alarm> rawData = null;
			var assembly = typeof(XMLReader).GetTypeInfo().Assembly;
			Stream stream = assembly.GetManifestResourceStream(this.path);

			await Task.Factory.StartNew(delegate
			{
				XDocument doc = XDocument.Load(stream);
				IEnumerable<Alarm> parameters = from s in doc.Root.Elements("Alarms").Descendants("Alarm")
												select new Alarm
												{
													EventCode = Convert.ToInt32(s.Attribute("EventCode").Value),
													Name = s.Element("Name").Value,
													Details = s.Element("Details").Value
												};

				rawData = parameters.ToList();
			});

			return rawData;
		}
	}
}
